/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package sync4j.syncclient.demo;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Frame;
import java.awt.Image;
import java.awt.Panel;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;

import java.text.SimpleDateFormat;

import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Properties;
import java.util.Vector;
import java.util.logging.Logger;

import sync4j.foundation.pdi.contact.Contact;
import sync4j.foundation.pdi.converter.Converter;
import sync4j.foundation.pdi.event.Calendar;
import sync4j.foundation.pdi.parser.ICalendarParser;
import sync4j.foundation.pdi.parser.VcardParser;
import sync4j.foundation.pdi.parser.XMLParser;

import sync4j.syncclient.spdm.DMException;
import sync4j.syncclient.spdm.ManagementNode;
import sync4j.syncclient.spdm.SimpleDeviceManager;

import sync4j.syncclient.common.SourceUtils;
import sync4j.syncclient.spds.SyncManager;

import com.funambol.commons.io.FileTools;

import org.xml.sax.SAXException;

/**
 * The main window for the SyncClient Demo GUI.
 *
 * @author Fabio Maggi @ Funambol
 * @author Alessandro Morandi, Giorgio Orsi
 * @version $Id: MainWindow.java,v 1.22.2.1 2005/01/26 16:03:20 fabius Exp $
 *
 */
public class MainWindow
extends Frame
implements ConfigurationParameters {

    // --- Folders Tree ---
    //
    // + (root)
    // |+ config
    // ||+ spds
    // |||+ sources
    // ||||- contact.properties
    // ||||- calendar.properties
    // |||
    // |||- syncml.properties
    // ||
    // ||+ xml
    // |||- init.xml
    // |||- map.xml
    // |||- mod.xml
    // ||
    // ||- application.properties
    // |
    // |+ db
    //  |+ contacts
    //  |+ calendar

    //---------------------------------------------------------- Constants

    public static final String DIR_CONTACTS           = "contacts"              ;
    public static final String DIR_CALENDAR           = "calendar"              ;
    public static final String DM_VALUE_PATH          = "spds/syncml"           ;
    public static final String DM_VALUE_CONTACT_PATH  = "spds/sources/contact"  ;
    public static final String DM_VALUE_CALENDAR_PATH = "spds/sources/calendar" ;

    public static final String PROP_CHARSET           = "spds.charset"          ;
    public static final String PROP_WD                = "wd"                    ;

    public static final String LOG_FILE_NAME          = "sync4j.pdi.syncclient" ;

    public static final String DATE_FORMAT            = "yyyyMMdd'T'HHmmss"     ;

    public static final String ROOT_DIRECTORY         = "config"                ;

    public static final String VALUE_UTF8             = "UTF8"                  ;

    public static final String XML_VERSION
        = "<?xml version=\"1.0\" encoding=\""                                   +
          System.getProperty("file.encoding")                                   +
          "\"?>"                                                                ;


    //---------------------------------------------------------- Protected data

    /**
     * A list of Contact objects
     */
    protected Vector contacts  ;

    /**
     * A list of Calendar objects
     */
    protected Vector calendars ;

    //---------------------------------------------------------- Private data

    /**
     * DM root node
     */
    private ManagementNode   rootNode            = null           ;

    /**
     * DM values from DM_VALUE_PATH
     */
    private Hashtable        syncmlValues        = null           ;

    /**
     * DM values from DM_VALUE_CONTACT_PATH
     */
    private Hashtable        xmlContactValues    = null           ;

    /**
     * DM values from DM_VALUE_CALENDAR_PATH
     */
    private Hashtable        xmlCalendarValues   = null           ;

    /**
     * Current index in the contacts Vector
     */
    private Integer          currentIndex        = null           ;

    /**
     * The logger wrapper
     */
    private Logger           logger              = null           ;

    /**
     * The root directory
     */
    private String           rootDirectory       = null           ;

    /**
     * The source directory
     */
    private String           sourceDirectory     = "db"           ;

    private CardLayout       cardLayout          = null           ;
    private Panel            mainPanel           = null           ;
    private DemoMenuBar      menubar             = null           ;
    private CalendarNew      calNew              = null           ;
    private CalendarList     calList             = null           ;
    private CalendarModify   calendarModify      = null           ;
    private ContactNew       contNew             = null           ;
    private ContactList      contList            = null           ;
    private ContactModify    contactModify       = null           ;

    private Synchronization  csync               = null           ;
    private Configuration    config              = null           ;

    private Language         ln                 =  new Language() ;

    //---------------------------------------------------------- Public methods

    /**
     * Creates a new main window. This constructor also initializes the logger,
     * the DeviceManager and fetches synchronization and contacts informations
     * from the filesystem
     */
    public MainWindow() {

        super    ();
        setTitle (ln.getString("sync4j_sync_client_pim_demo"));

        Image icon = Toolkit.getDefaultToolkit().getImage(FRAME_ICONNAME);

        setIconImage(icon);

        Panel       bottomPanel = null ;
        Properties  props       = null ;

        contacts  = new Vector () ;
        calendars = new Vector () ;

        rootDirectory = System.getProperty(PROP_WD);
        props = System.getProperties();
        if (System.getProperty(SimpleDeviceManager.PROP_DM_DIR_BASE) == null) {
            props.put(SimpleDeviceManager.PROP_DM_DIR_BASE ,
                      buildPath(rootDirectory, ROOT_DIRECTORY));
            System.setProperties(props);
        }
        props = System.getProperties();
        props.put(PROP_CHARSET, VALUE_UTF8);
        System.setProperties(props);

        sourceDirectory = buildPath(rootDirectory,sourceDirectory);

        //
        // --- Logger Setup ---
        //
        logger = Logger.getLogger(LOG_FILE_NAME);

        //
        // --- Device Manager ---
        //
        rootNode = SimpleDeviceManager.getDeviceManager().getManagementTree();

        try {
            syncmlValues      = rootNode.getNodeValues ( DM_VALUE_PATH          ) ;
            xmlContactValues  = rootNode.getNodeValues ( DM_VALUE_CONTACT_PATH  ) ;
            xmlCalendarValues = rootNode.getNodeValues ( DM_VALUE_CALENDAR_PATH ) ;
        } catch (DMException exc) {
            logger.severe(ln.getString ("cannot_find_devicemanager_values") +
                          " "                                               +
                          exc.getMessage()                                  );
        }

        //
        // --- Layout Creation ---
        //
        setSize(440,495);
        setLocation(400,200);

        setLayout(new BorderLayout());
        mainPanel = new Panel();

        //
        // A little trick to add blank space on the bottom of the window
        //
        bottomPanel = new Panel();
        bottomPanel.setSize(240, 5);

        cardLayout = new CardLayout();
        mainPanel.setLayout(cardLayout);

        menubar  = new DemoMenuBar(this) ;
        contList = new ContactList(this) ;

        refresh();

        contactModify     = new ContactModify   (this)  ;
        calendarModify    = new CalendarModify  (this) ;
        contNew           = new ContactNew      (this) ;
        csync             = new Synchronization (this) ;
        config            = new Configuration   (this) ;

        calList     = new CalendarList(this);
        calNew      = new CalendarNew(this);

        //
        // Add a menu and all the components to the card layout
        //
        setMenuBar(menubar);

        mainPanel.add(contList       , KEY_CONTACTLIST    ) ;
        mainPanel.add(contactModify  , KEY_CONTACTMODIFY  ) ;
        mainPanel.add(calendarModify , KEY_CALENDARMODIFY ) ;
        mainPanel.add(contNew        , KEY_CONTACTNEW     ) ;
        mainPanel.add(csync          , KEY_SYNC           ) ;
        mainPanel.add(config         , KEY_CONFIG         ) ;
        mainPanel.add(calList        , KEY_CALENDARLIST   ) ;
        mainPanel.add(calNew         , KEY_CALENDARNEW    ) ;

        add(mainPanel,BorderLayout.CENTER);
        add(bottomPanel,BorderLayout.SOUTH);

        // Default card
        cardLayout.show(mainPanel, KEY_CONTACTLIST);

        // Add the listeners
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent evt) {
                exit();
            }
        } );
    }

    //---------------------------------------------------------- Protected methods

    /**
     * Closes the window and stops the application
     */
    protected void exit() {

        //
        // hide the Frame
        //
        setVisible(false);

        dispose();

        //
        // tell windowing system to free resources
        //
        logger.fine(ln.getString ("logging_stopped"));

        System.exit(0);
    }

    /**
     * Manages the contact synchronization, based on the configuration parameters
     * provided by the DeviceManager
     */
    protected void synchronize() {

        csync.setButton(false);

        try {
            csync.log(ln.getString ("setting_up_sync_manager"));
            SyncManager syncManager = SyncManager.getSyncManager("");

            csync.log(ln.getString ("synchronizing"));
            syncManager.sync();

            csync.log(ln.getString ("done"));
            csync.setButton(true);
        } catch (Exception exc) {
            if (exc.getMessage()==null) {
                csync.log(ln.getString ("synch_exception") +
                                        " "                +
                          exc                              );
            } else {
                csync.log(exc.getMessage());
            }
            logger.throwing(this.getClass().getName(),"synchronize",exc);
            csync.setButton(true);
        }
    }

    /**
     * Refreshes the contact list reading the local vCard files
     */
    protected void refresh() {
        contacts = getContacts();
        contList.fillList();
        show(KEY_CONTACTLIST);
    }
    /**
     * Refreshes the calendar list reading the local calendar files
     */
    protected void refreshCalendar() {
        calendars = getCalendars();
        calList.fillList();
    }

    /**
     * Refreshes the contact list reading the local contact files
     */
    protected void refreshContact() {

        contacts  = getContacts() ;

        contacts  = sortContacts(contacts);

        contList.fillList();
    }

    /**
     * Shows the specified card in the CardLayout.
     * When showing the contact modification card, the method fills all the fields.
     * When showing the configuration card, the method fills all the fields.
     * When showing the new contact card, the method blanks all the fields.
     * When showing the synchronization card, the method starts a new synchronization.
     *
     * @param card the card to be shown
     */
    protected void show(String card) {

        cardLayout.show(mainPanel,card);

        if (card.equals         (KEY_CALENDARLIST    ))      {

            refreshCalendar();

        } else if (card.equals  (KEY_CALENDARMODIFY  ))      {

            calendarModify.fillFields(((DemoCalendar) calendars.
                elementAt(getCurrentIndex().intValue())).getCalendar());

        } else if (card.equals  (KEY_CALENDARNEW     ))      {

            calNew.blankFields();

        } else if (card.equals  (KEY_CONFIG           ))     {

            config.setAllFields
                (syncmlValues, xmlContactValues, xmlCalendarValues);

        } else if (card.equals  (KEY_CONTACTLIST     ))      {

            refreshContact();

        } if (card.equals       (KEY_CONTACTMODIFY   ))      {

            contactModify.fillFields(((DemoContact) contacts.
                elementAt(getCurrentIndex().intValue())).getContact());

        } else if (card.equals  (KEY_CONTACTNEW      ))      {

            contNew.blankFields();

        } else if (card.equals  (KEY_SYNC            ))      {

            synchronize();

        }

    }

    /**
     * Returns a vector of Contact elements containing the list of contacts
     * found in the sourceDirectory
     *
     * @return a Vector of Contacts, or an empty vector if no contacts are found
     */
    protected Vector getContacts() {

        try {

            File        curFile        = null ;

            Vector      existingFiles  = null ;
            Vector      contacts       = null ;
            VcardParser vcp            = null ;
            Contact     contact        = null ;

            contacts = new Vector();

            existingFiles = getExistingFiles(DIR_CONTACTS);

            for (int i=0, l = existingFiles.size(); i < l; i++) {

                curFile = (File) existingFiles.elementAt(i);

                try {

                    logger.finest(ln.getString ("parsing_file") +
                                  " "                           +
                                  curFile                       );

                    XMLParser xmlParser = new XMLParser();

                    contact = (Contact) xmlParser.
                        parse(new FileInputStream(curFile));
                    contacts.addElement
                        (new DemoContact(curFile.getName(),contact));

                } catch (SAXException e) {
                    logger.warning(ln.getString ("error_sax_exception") +
                                   " "                                  +
                                   curFile                              +
                                   ": "                                 +
                                   e.getMessage()                       );
                }

            }

            return contacts;
        } catch (IOException exc) {
            logger.severe(ln.getString ("error_reading_files") +
                          " "                                  +
                          exc.getMessage()                     );
        }

        return new Vector();
    }

    /**
     * Returns a vector of Calendar elements containing the list of calendar
     * found in the sourceDirectory
     *
     * @return a Vector of Calendar, or an empty vector if no calendars are found
     */
    protected Vector getCalendars() {

        try {

            File            curFile       = null ;

            Vector          existingFiles = null ;
            Vector          calendars     = null ;
            ICalendarParser icp           = null ;
            Calendar        calendar      = null ;

            calendars = new Vector();

            existingFiles = getExistingFiles(DIR_CALENDAR);
            for (int i=0, l = existingFiles.size(); i < l; i++) {

                curFile = (File) existingFiles.elementAt(i);

                try {
                    logger.finest(ln.getString ("parsing_file") +
                                  " "                           +
                                  curFile                       );

                    XMLParser xmlParser = new XMLParser();

                    calendar = (Calendar) xmlParser.
                        parse(new FileInputStream(curFile));
                    calendars.addElement
                        (new DemoCalendar(curFile.getName(), calendar));
                } catch (SAXException e) {
                    logger.warning(ln.getString ("error_sax_exception") +
                                   " "                                  +
                                   curFile                              +
                                   ": "                                 +
                                   e.getMessage()                       );
                }

            }

            return calendars;

        } catch (IOException exc) {
            logger.severe(ln.getString ("error_reading_files") +
                                        " "                    +
                                        exc.getMessage()       );
        }
        return new Vector();
    }

    /**
     * Overwrites the indexth contact with the specified contact
     *
     * @param contact the new contact
     * @param index the index of the contact to be overwritten
     */
    protected void writeModContact(Contact contact, int index) {

        File              tmpFile          = null ;
        File              f                = null ;
        FileOutputStream  out              = null ;
        PrintStream       ps               = null ;

        String            output           = null ;

        HashMap           hashMapFromFile  = null ;
        HashMap           hashMap          = null ;

        try {

            SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT);

            contact.setRevision(sdf.format(new Date()));

            Converter conv = new Converter();

             //
            // Conversion into XML. Modified way
            //
            output = conv.contactToXML(contact);

            f = new File(buildPath(sourceDirectory,DIR_CONTACTS),
                         ((DemoContact) contacts.
                            elementAt(getCurrentIndex().intValue())).
                                                        getFilename());

            hashMapFromFile = SourceUtils.xmlToHashMap(new FileInputStream(f));

            hashMap = SourceUtils.xmlToHashMap(output);

            hashMapFromFile.putAll(hashMap);

            //
            // Delete old file
            //
            f.delete();

            //
            // Modify the contacts Vector
            //
            ((DemoContact) contacts.elementAt(index)).setContact(contact);
            contList.fillList();

            //
            // Write new file (same name as the old one)
            //
            tmpFile = new File(buildPath(sourceDirectory,DIR_CONTACTS),
                                ((DemoContact) contacts.
                                        elementAt(index)).getFilename());

            out = new FileOutputStream (tmpFile ) ;
            ps  = new PrintStream      (out     ) ;

            String value = SourceUtils.hashMapToXml(hashMapFromFile);

            if (value.indexOf(XML_VERSION) != -1) {
                ps.print(value);
            } else                                {
                ps.print(XML_VERSION + value);
            }

        } catch (IOException exc) {
            logger.severe(ln.getString ("error_updating_file") +
                          " "                                  +
                          exc.getMessage());
        } catch (Exception exc) {
            logger.throwing(getClass().getName() ,
                            "writeModContact"    ,
                            exc                  );
            logger.severe(ln.getString ("error_updating_file") +
                                        " "                    +
                                        exc.getMessage()       );
        } finally {
            if (ps != null) {
                try { ps.close();  } catch (Exception e) { }
                ps = null;
            }
            if (out != null) {
                try { out.close(); } catch (Exception e) { }
                out = null;
            }
        }
    }

    /**
     * Creates a new contact which will be inserted in the contacts list
     *
     * @param contact the new contact
     */
    protected void writeNewContact(Contact contact) {

        Converter        conv     = null ;

        SimpleDateFormat sdf      = null ;
        Date             tmpDate  = null ;

        String           fileName = null ;

        String           output   = null ;

        File             tmpFile  = null ;
        FileOutputStream out      = null ;
        PrintStream      ps       = null ;

        try {

            sdf     = new SimpleDateFormat(DATE_FORMAT);
            tmpDate = new Date();
            contact.setRevision(sdf.format(tmpDate));

            conv = new Converter();

            fileName = String.valueOf(tmpDate.getTime());

            //
            // Conversion into XML. Modified way
            //
            output = conv.contactToXML(contact);

            //
            // Add to the contacts Vector
            //
            contacts.addElement(new DemoContact(fileName, contact));
            contList.fillList();

            //
            // Write to filesystem
            //
            tmpFile = new File(buildPath(sourceDirectory, DIR_CONTACTS) ,
                               fileName                                 );

            out = new FileOutputStream (tmpFile ) ;
            ps  = new PrintStream      (out     ) ;

            //
            // work around about SyncConnectorDB, ID tag must be in message
            //
            output = output.substring(0, output.indexOf ("</contact>")) +
                     "<id>"                                             +
                     fileName                                           +
                     "</id>"                                            +
                     output.substring(output.indexOf    ("</contact>")) ;

            ps.print(output);

        } catch (IOException exc) {
            logger.severe(ln.getString ("error_creating_file") +
                          " "                                  +
                          exc.getMessage()                     );
        } finally {
            if (ps != null) {
                try { ps.close();  } catch (Exception e) { }
                ps = null;
            }
            if (out != null) {
                try { out.close(); } catch (Exception e) { }
                out = null;
            }
        }
    }

    /**
     * Deletes the currently selected contact
     */
    protected void deleteContact() {

        try {
            new File(buildPath(sourceDirectory,DIR_CONTACTS),
                     ((DemoContact) contacts.
                        elementAt(getCurrentIndex().intValue())).
                                                getFilename()).delete();
            refresh();
        } catch (NullPointerException exc) {
            logger.severe(ln.getString ("error_deleting_file") +
                          " "                                  +
                          exc.getMessage()                     );
        }
    }

    /**
     * Deletes the currently selected calendar
     */
    protected void deleteCalendar() {
        try {
            new File(buildPath(sourceDirectory,DIR_CALENDAR),
                     ((DemoCalendar) calendars.
                        elementAt(getCurrentIndex().intValue())).
                                                getFilename()).delete();
            refreshCalendar();
        } catch (NullPointerException exc) {
            logger.severe(ln.getString ("error_deleting_file") +
                          " "                                  +
                          exc.getMessage());
        }
    }

    /**
     * Creates a new contact which will be inserted in the contacts list
     *
     * @param calendar the new contact
     */
    protected void writeNewCalendar(Calendar calendar) {

        Converter        conv     = null ;

        SimpleDateFormat sdf      = null ;
        Date             tmpDate  = null ;

        String           output   = null ;
        String           fileName = null ;

        File             tmpFile  = null ;
        FileOutputStream out      = null ;
        PrintStream      ps       = null ;

        try {

            sdf     = new SimpleDateFormat(DATE_FORMAT);
            tmpDate = new Date();

            fileName = String.valueOf(tmpDate.getTime());

            conv = new Converter();

            //
            // Conversion into XML. Modified way
            //
            output = conv.calendarToXML(calendar);

            //
            // Add to the calendars Vector
            //
            calendars.addElement(new DemoCalendar(fileName, calendar));
            calList.fillList();

            //
            // Write to filesystem
            //
            tmpFile = new File(buildPath(sourceDirectory, DIR_CALENDAR ),
                               fileName                                );

            out = new FileOutputStream (tmpFile );
            ps  = new PrintStream      (out     );

            //
            // work around about SyncConnectorDB, ID tag must be in message
            //
            output = output.substring(0, output.indexOf ("</appointment>")) +
                     "<id>"                                                 +
                     fileName                                               +
                     "</id>"                                                +
                     output.substring(output.indexOf    ("</appointment>")) ;

            ps.print(output);

        }  catch (IOException exc) {
            logger.severe(ln.getString ("error_creating_file") +
                          " "                                  +
                          exc.getMessage()                     );
        }  finally {
            if (ps != null) {
                try { ps.close();  } catch (Exception e) { }
                ps = null;
            }
            if (out != null) {
                try { out.close(); } catch (Exception e) { }
                out = null;
            }
        }

    }

    /**
     * Overwrites the indexth contact with the specified contact
     *
     * @param calendar the new contact
     * @param index the index of the contact to be overwritten
     */
    protected void writeModCalendar(Calendar calendar, int index) {

        Converter        conv             = null ;

        String           output           = null ;

        HashMap          hashMapFromFile  = null ;
        HashMap          hashMap          = null ;

        File             f                = null ;

        File             tmpFile          = null ;
        FileOutputStream out              = null ;
        PrintStream      ps               = null ;

        try {

            conv = new Converter();

            //
            // Conversion into XML. Modified way
            //
            output = conv.calendarToXML(calendar);


            f = new File(buildPath(sourceDirectory, DIR_CALENDAR),
                         ((DemoCalendar) calendars.elementAt(
                           getCurrentIndex().intValue())).getFilename());

            hashMapFromFile = SourceUtils.xmlToHashMap(FileTools.readTextFile(f));

            hashMap = SourceUtils.xmlToHashMap(output);

            hashMapFromFile.putAll(hashMap);

            //
            // Delete old file(
            //
            f.delete();

            //
            // Modify the contacts Vector
            //
            ((DemoCalendar) calendars.elementAt(index)).setCalendar(calendar);
            contList.fillList();

            //
            // Write new file (same name as the old one)
            //
            tmpFile = new File(buildPath(sourceDirectory,DIR_CALENDAR),
                               ((DemoCalendar) calendars.
                                    elementAt(index)).getFilename());

            out = new FileOutputStream (tmpFile ) ;
            ps =  new PrintStream      (out     ) ;

            String value = SourceUtils.hashMapToXml(hashMapFromFile);

            if (value.indexOf(XML_VERSION) != -1) {
                ps.print(value);
            } else                                {
                ps.print(XML_VERSION + value);
            }

        } catch (IOException exc) {
            logger.severe(ln.getString ("error_updating_file") +
                          " "                                  +
                          exc.getMessage()                     );
        } catch (Exception exc) {
            logger.severe(ln.getString ("error_updating_file") +
                          " "                                  +
                          exc.getMessage()                     );
        } finally {
            if (ps != null) {
                try { ps.close  (); } catch (Exception e) { }
                ps = null;
            }
            if (out != null) {
                try { out.close (); } catch (Exception e) { }
                out = null;
            }
        }
    }

    /**
     * Saves the specified configuration parameters
     * by passing them to the DeviceManager
     *
     * @param values a list of configuration values
     */
    protected void writeConfig(Hashtable values) {

        try {
            logger.fine(ln.getString ("writing_configuration"));

            Configuration cf = null;

            syncmlValues.put (PARAM_SYNCMLURL    ,
                              values.get( PARAM_SYNCMLURL    ) );
            syncmlValues.put (PARAM_USERNAME     ,
                              values.get( PARAM_USERNAME     ) );
            syncmlValues.put (PARAM_PASSWORD     ,
                              values.get( PARAM_PASSWORD     ) );
            syncmlValues.put (PARAM_DEVICEID     ,
                              values.get( PARAM_DEVICEID     ) );
            syncmlValues.put (PARAM_ADJUSTDATE   ,
                              values.get( PARAM_ADJUSTDATE   ) ) ;
            syncmlValues.put (PARAM_MESSAGETYPE  ,
                              values.get( PARAM_MESSAGETYPE  ) ) ;
            syncmlValues.put (PARAM_LOGLEVEL     ,
                              values.get( PARAM_LOGLEVEL     ) ) ;

            xmlCalendarValues.put(PARAM_SYNCSOURCEURI,values.
                                get(PARAM_SOURCEURICALENDAR  ) ) ;
            xmlCalendarValues.put(PARAM_SYNCMODE, values.
                                get(PARAM_SYNCCALENDAR       ) ) ;
            xmlContactValues.put (PARAM_SYNCSOURCEURI, values.
                                get(PARAM_SOURCEURICONTACT   ) ) ;
            xmlContactValues.put (PARAM_SYNCMODE, values.
                                get(PARAM_SYNCCONTACT        ) ) ;

            rootNode.setValue(DM_VALUE_PATH                  ,
                              PARAM_TARGETLOCALURI           ,
                              getBaseURL((String) values.
                                    get(PARAM_SYNCMLURL)          ) ) ;
            rootNode.setValue(DM_VALUE_PATH                  ,
                              PARAM_SYNCMLURL                ,
                              values.get( PARAM_SYNCMLURL         ) ) ;
            rootNode.setValue(DM_VALUE_PATH                  ,
                              PARAM_USERNAME                 ,
                              values.get( PARAM_USERNAME          ) ) ;
            rootNode.setValue(DM_VALUE_PATH                  ,
                              PARAM_PASSWORD                 ,
                              values.get( PARAM_PASSWORD          ) ) ;
            rootNode.setValue(DM_VALUE_CONTACT_PATH          ,
                              PARAM_SYNCSOURCEURI            ,
                              values.get( PARAM_SOURCEURICONTACT  ) ) ;
            rootNode.setValue(DM_VALUE_CONTACT_PATH          ,
                              PARAM_SYNCMODE            ,
                              values.get( PARAM_SYNCCONTACT       ) ) ;
            rootNode.setValue(DM_VALUE_CALENDAR_PATH         ,
                              PARAM_SYNCSOURCEURI            ,
                              values.get( PARAM_SOURCEURICALENDAR ) ) ;
            rootNode.setValue(DM_VALUE_CALENDAR_PATH         ,
                              PARAM_SYNCMODE                 ,
                              values.get( PARAM_SYNCCALENDAR      ) ) ;
            rootNode.setValue(DM_VALUE_PATH                  ,
                              PARAM_DEVICEID                 ,
                              values.get( PARAM_DEVICEID          ) ) ;
            rootNode.setValue(DM_VALUE_PATH                  ,
                              PARAM_ADJUSTDATE               ,
                              values.get( PARAM_ADJUSTDATE        ) ) ;
            rootNode.setValue(DM_VALUE_PATH                  ,
                              PARAM_MESSAGETYPE              ,
                              values.get( PARAM_MESSAGETYPE       ) ) ;
            rootNode.setValue(DM_VALUE_PATH                  ,
                              PARAM_LOGLEVEL                 ,
                              values.get( PARAM_LOGLEVEL          ) ) ;


        } catch (DMException exc) {
            logger.severe(ln.getString ("device_manager_error") +
                          " "                                   +
                          exc.getMessage ()                     );

        } catch (Exception exc) {
            logger.severe(ln.getString ("unhandled_exception")  +
                          " "                                   +
                          exc.toString   ()                     );
        }
    }

    /**
     * Returns the index of the currently selected contact
     *
     * @return the index of the currently selected contact
     */
    protected Integer getCurrentIndex() {
        return currentIndex;
    }

    /**
     * Sets the current index, thus selecting a contact
     *
     * @param index the value the current index should be set to
     */
    protected void setCurrentIndex(Integer index) {
        currentIndex = index;
    }

    //---------------------------------------------------------- Private methods

    /**
     * Returns a string representing the path of the specified dir
     * concatenating it with the specified source directory
     *
     * @param sourceDirectory the source directory
     * @param dir the directory
     * @return the directory's path
     */
    private String buildPath(String sourceDirectory, String dir) {
        return sourceDirectory+"/"+dir;
    }

    /**
     * Returns a vector of File elements containing the list of files in the
     * sourceDirectory
     *
     * @return a list of files, or an empty vector if the directory is empty
     */
    private Vector getExistingFiles(String dir) throws IOException {
        Vector ret = new Vector();

        //File[] files = new File(buildPath(sourceDirectory,dir)).listFiles();
        String[] fileNames = new File(buildPath(sourceDirectory,dir)).list();
        if (fileNames != null) {
            for (int i = 0; i<fileNames.length; ++i) {
                ret.addElement(new File(buildPath(sourceDirectory,dir) ,
                               fileNames[i])                           );
            }
        }

        return ret;
    }

    private String getBaseURL (String url) {

        int firstSlash  = url.indexOf("/"                  ) ;
        int secondSlash = url.indexOf("/", firstSlash  + 1 ) ;
        int thirdSlash  = url.indexOf("/", secondSlash + 1 ) ;
        int portColon   = url.indexOf(":", secondSlash + 1 ) ;

        if (thirdSlash < portColon || portColon == -1) {

            //
            // No port is specified (ex: http://127.0.0.1/sync4j)
            //
            return url.substring(0, thirdSlash);
        } else {
            //
            // A port is specified and must be removed
            // (ex: http://127.0.0.1:8080/sync4j)
            //
            return url.substring(0, portColon);
        }
    }

    /**
     * sort contact
     *
     * @ param contacts to sort
     * @ return sorthed contacts
     */
    private Vector sortContacts(Vector contacts) {

        ContactComparator  contactComparator = null ;
        DemoContact[]      dcs               = null ;

        if (!(contacts.size () > 0)) {
             return contacts;
        }

        dcs = new DemoContact[contacts.size()];

        for (int i=0, l = contacts.size(); i < l; i++) {
            dcs[i] = (DemoContact) contacts.elementAt(i);
        }

        contactComparator = new ContactComparator () ;

        Arrays.sort(dcs, contactComparator);

        contacts = new Vector();

        for (int i=0, l = dcs.length; i < l; i++) {
            contacts.add(dcs[i]);
        }

        return contacts;

    }

}